@@ -169,14 +169,23 @@ class linenumber(commands.Command):
 		group.add_argument("--old-left", action='store_const', const=cls.model.linenumber_old_left, dest='_fmt', help=_("Show the line numbers before the commit (in case of a combined diff: the left version)"))
 		group.add_argument("--old-right", action='store_const', const=cls.model.linenumber_old_right, dest='_fmt', help=_("Show the line numbers before the commit (in case of a combined diff: the right version)"))
 		group.add_argument("--toggle", action='store_true', help=_("Toggle between --old and --new"))
+		group.add_argument("--cycle", action='store_true', help=_("Cycle between --new, --old and --off"))
 		group.add_argument("--off", action='store_const', const=cls.model.linenumber_off, dest='_fmt', help=_("Do not show line numbers"))
 
 	def execute(self, args):
 		if args.toggle:
 			args.fmt = self.model.linenumber_new if self.model.linenumber==self.model.linenumber_old else self.model.linenumber_old
+		elif args.cycle:
+			if self.model.linenumber == self.model.linenumber_old:
+				args.fmt = self.model.linenumber_off
+			elif self.model.linenumber == self.model.linenumber_new:
+				args.fmt = self.model.linenumber_old
+			else:
+				args.fmt = self.model.linenumber_new
 		else:
 			args.fmt = args._fmt
 		self.app.line_numbers(args.fmt)
+		self.app.show_info('linenumber format changed to %r' % args.fmt)
 
 class load(commands.Command):
 
