@@ -99,7 +99,7 @@ def command_function(
 # ---------- model classes ----------
 
 class NameSpace:
-	pass
+	is_line_counting_possible: 'bool|None'
 
 class FileInfo:
 
@@ -813,6 +813,8 @@ class DetailsModel(Model):
 	linenumber = linenumber_new
 	min_line_number_width = 3
 
+	ANSI_RED = "[31m"
+	ANSI_GREEN = "[32m"
 	re_ansi_esscape_sequence = "(\[[0-9;]*m)?"
 	reo_start_new_file = re.compile(re_ansi_esscape_sequence + "diff (--git \"?a/(?P<fn1>.*?)\"? \"?b/|--cc \"?(?P<fn2>[^\"]*)\"?|--combined \"?(?P<fn3>[^\"]*))")
 	reo_escape = re.compile(r'(\\[0-7][0-7][0-7])+')
@@ -984,6 +986,7 @@ class DetailsModel(Model):
 		ns = NameSpace()
 		ns.out = out
 		ns.look_for_hint = True
+		ns.is_line_counting_possible = None
 		for ns.i in range(len(ns.out)):
 			ln = ns.out[ns.i]
 			if isinstance(ln, tuple):
@@ -1066,8 +1069,9 @@ class DetailsModel(Model):
 		ns.line_info.linenumber_after = ns.linenumber_after
 		ns.line_info.linenumber_before_right = ns.linenumber_before_right
 		ns.line_info.linenumber_before_left = ns.linenumber_before_left
-		ns.line_info.is_modified = False
+		ns.line_info.is_modified = self.ANSI_RED in ln or self.ANSI_GREEN in ln
 		no_increment = True
+		all_incremented = True
 
 		if ns.reo_line_exists_after.match(ln):
 			ns.line_info.formatted_linenumber_after = ns.fmt_linenumber_after % ns.linenumber_after
@@ -1075,7 +1079,7 @@ class DetailsModel(Model):
 			no_increment = False
 		else:
 			ns.line_info.formatted_linenumber_after = ns.fmt_linenumber_after % ""
-			ns.line_info.is_modified = True
+			all_incremented = False
 
 		if ns.reo_line_existed_left.match(ln):
 			ns.line_info.formatted_linenumber_before_left = ns.fmt_linenumber_before_left % ns.linenumber_before_left
@@ -1083,7 +1087,7 @@ class DetailsModel(Model):
 			no_increment = False
 		else:
 			ns.line_info.formatted_linenumber_before_left = ns.fmt_linenumber_before_left % ""
-			ns.line_info.is_modified = True
+			all_incremented = False
 
 		if ns.reo_line_existed_right.match(ln):
 			ns.line_info.formatted_linenumber_before_right = ns.fmt_linenumber_before_right % ns.linenumber_before_right
@@ -1091,31 +1095,73 @@ class DetailsModel(Model):
 			no_increment = False
 		else:
 			ns.line_info.formatted_linenumber_before_right = ns.fmt_linenumber_before_right % ""
-			ns.line_info.is_modified = True
+			all_incremented = False
 
 		if no_increment:
+			# None of the regular expressions has matched.
 			# The usual way of counting lines does not work.
 			# This happens with --word-diff.
 			# So I'm assuming each line is present in all versions.
 			# Of course this is not always fullfilled.
 			# But it's still better than giving each line the same number.
+			ns.is_line_counting_possible = False
+			self.remove_last_line_numbers(ns)
 			ns.line_info.formatted_linenumber_after = ""
 			ns.line_info.formatted_linenumber_before_left = ""
 			ns.line_info.formatted_linenumber_before_right = ""
 			ns.linenumber_after += 1
 			ns.linenumber_before_left += 1
 			ns.linenumber_before_right += 1
+
+			# Because line counting does not work it is possible
+			# that I have reached the start of the next file already.
+			# I have copied the logic from insert_linenumbers__process_metainfo_line.
+			# I cannot easily factor it out into a new method check_new_file
+			# because insert_linenumbers__process_content_line handles
+			# ln and ns.out[ns.i] too differently.
+			if self.is_start_new_file(ln):
+				if self.file_info.fn != self.file_info.original_fn:
+					ln = ln.replace(self.file_info.original_fn, self.file_info.fn)
+				if self.insert_align_character:
+					ln = ALIGN_CHAR + ln
+				ln = (ln, TYPE_START_OF_FILE, self.file_info)
+				ns.out[ns.i] = ln
+				ns.look_for_hint = True
+				return
+
 			if self.insert_align_character:
 				ln = ALIGN_CHAR + ln
 		elif self.insert_align_character:
 			ln = self.insert_align_char(ln, ns.i_align_character)
 
+		if ns.is_line_counting_possible is None:
+			if not all_incremented:
+				ns.is_line_counting_possible = True
+			elif ns.line_info.is_modified:
+				ns.is_line_counting_possible = False
+				self.remove_last_line_numbers(ns)
+		elif ns.is_line_counting_possible is False:
+			ns.line_info.formatted_linenumber_after = ""
+			ns.line_info.formatted_linenumber_before_left = ""
+			ns.line_info.formatted_linenumber_before_right = ""
+
 		if ns.linenumber_after > ns.last_linenumber_after and ns.linenumber_before_right > ns.last_linenumber_before_right and ns.linenumber_before_left > ns.last_linenumber_before_left:
 			ns.look_for_hint = True
 
 		ln = (ln, TYPE_NUMBERED_LINE, self.indent_broken_code, ns.line_info)
 		ns.out[ns.i] = ln
 
+	def remove_last_line_numbers(self, ns) -> None:
+		for i in range(ns.i-1, 0, -1):
+			ln = ns.out[i]
+			if isinstance(ln, tuple) and len(ln) > 1 and ln[1] == TYPE_NUMBERED_LINE:
+				line_info = ln[-1]
+				line_info.formatted_linenumber_after = ""
+				line_info.formatted_linenumber_before_right = ""
+				line_info.formatted_linenumber_before_left = ""
+			else:
+				break
+
 	@staticmethod
 	def insert_align_char(ln, pos_align_character):
 		n_escape = 0
